"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
var template_1 = __importDefault(require("@babel/template"));
var uniqBy_1 = __importDefault(require("lodash/uniqBy"));
var utils_1 = require("./utils");
var const_1 = require("./const");
var visitor_1 = require("./visitor");
var getInitState = function () { return ({
    needTransform: false,
    reactSpecifier: [],
    compNames: []
}); };
/**
 * Plugin Instance
 */
var RuiPlugin = /** @class */ (function () {
    function RuiPlugin(types, opts) {
        this.state = getInitState();
        this.types = types;
        this.opts = (0, utils_1.mergeOptions)(opts);
    }
    RuiPlugin.prototype.getState = function () {
        return this.state;
    };
    RuiPlugin.prototype.updateState = function (key, payload) {
        if (typeof payload === 'function') {
            this.state[key] = payload(this.state[key]);
        }
        else {
            this.state[key] = payload;
        }
    };
    RuiPlugin.prototype.resetState = function () {
        this.state = getInitState();
    };
    /** First inspect, determine if need transform */
    RuiPlugin.prototype.inspect = function (path) {
        path.traverse(visitor_1.entryVisitor, { plugin: this });
    };
    /** Add all react import in the end */
    RuiPlugin.prototype.addReactImport = function (path) {
        var t = this.types;
        var _a = this.state, needTransform = _a.needTransform, reactSpecifier = _a.reactSpecifier;
        if (needTransform) {
            var defaultSpecifier = t.importDefaultSpecifier(t.identifier('React'));
            var extraSpecifier = const_1.reactSpecifiersName.map(function (n) {
                return t.importSpecifier(t.identifier(n), t.identifier(n));
            });
            var uniqSpecifier = (0, uniqBy_1["default"])(reactSpecifier.concat(extraSpecifier), function (i) {
                return i.local.name;
            });
            var importDeclaration = t.importDeclaration(__spreadArray([defaultSpecifier], uniqSpecifier, true), t.stringLiteral('react'));
            path.unshiftContainer('body', importDeclaration);
        }
    };
    /** Overwrite AST related to components */
    RuiPlugin.prototype.overwriteComponents = function (path) {
        var t = this.types;
        var compNames = this.state.compNames;
        var _a = this.opts, splitChunkByComp = _a.splitChunkByComp, chunkNames = _a.chunkNames, libraryName = _a.libraryName, _b = _a.libraryDir, pcLibraryDir = _b.pc, mLibraryDir = _b.mobile;
        path.traverse({
            ImportDeclaration: function (path) {
                // last import
                if (!t.isImportDeclaration(path.getNextSibling())) {
                    //依次追加components
                    compNames.forEach(function (name) {
                        var getAST = (0, template_1["default"])(const_1.componentTemplate, {
                            plugins: ['jsx'],
                            // 仅可保留表达式外的注释
                            preserveComments: true
                        });
                        var mName = "M".concat(name);
                        var pcName = "PC".concat(name);
                        // template不支持innerComment，手动添加ast
                        var mLibName = typeof mLibraryDir === 'function' ? mLibraryDir((0, utils_1.toKebabCase)(name)) : mLibraryDir;
                        var mImport = t.callExpression(t.identifier('import'), [
                            (0, utils_1.addChunkNameComment)(t.stringLiteral("".concat(libraryName, "/").concat(mLibName)), splitChunkByComp ? "".concat(chunkNames.mobile, "-").concat(name) : chunkNames.mobile),
                        ]);
                        var pcLibName = typeof pcLibraryDir === 'function' ? pcLibraryDir((0, utils_1.toKebabCase)(name)) : pcLibraryDir;
                        var pcImport = t.callExpression(t.identifier('import'), [
                            (0, utils_1.addChunkNameComment)(t.stringLiteral("".concat(libraryName, "/").concat(pcLibName)), splitChunkByComp ? "".concat(chunkNames.pc, "-").concat(name) : chunkNames.pc),
                        ]);
                        path.insertAfter(getAST({
                            COMPONENT_NAME: t.identifier(name),
                            M_COMP_VAR: t.identifier(mName),
                            M_COMP_TAG: t.jsxIdentifier(mName),
                            M_IMPORT: mImport,
                            PC_COMP_VAR: t.identifier(pcName),
                            PC_COMP_TAG: t.jsxIdentifier(pcName),
                            PC_IMPORT: pcImport
                        }));
                    });
                }
            }
        });
    };
    return RuiPlugin;
}());
exports["default"] = RuiPlugin;
